# pylint: disable-msg=W0622
"""cubicweb-card application packaging information"""

modname = 'cubicweb_card'
distname = 'cubicweb-card'

numversion = (1, 0, 1)
version = '.'.join(str(num) for num in numversion)

license = 'LGPL'
author = 'LOGILAB S.A. (Paris, FRANCE)'
author_email = 'contact@logilab.fr'
web = 'http://www.cubicweb.org/project/%s' % distname
description = 'card/wiki component for the CubicWeb framework'
classifiers = [
           'Environment :: Web Environment',
           'Framework :: CubicWeb',
           'Programming Language :: Python',
           'Programming Language :: JavaScript',
]

__depends__ = {
    'cubicweb': '>= 3.24',
    'docutils': None,
    }
__recommends__ = {
    'cubicweb-preview': None,
    'cubicweb-seo': None,
    }



# copyright 2013 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

"""test on entities"""

from logilab.common.testlib import unittest_main
from cubicweb.devtools.testlib import CubicWebTC


class DCTitleTC(CubicWebTC):
    def test_nowikiid(self):
        with self.admin_access.client_cnx() as cnx:
            venus = cnx.create_entity('Card', title=u"Venus",
                                      content=u"a planet")
            self.assertEqual(venus.dc_title(), 'Venus')

    def test_wikiid(self):
        with self.admin_access.client_cnx() as cnx:
            venus = cnx.create_entity(
                'Card', title=u"Venus",
                content=u"a planet", wikiid=u'Planet_Venus')
            self.assertEqual(venus.dc_title(), 'Planet_Venus')


class BreadCrumbsTC(CubicWebTC):
    def test_onecard_nowikiid(self):
        with self.admin_access.web_request() as req:
            venus = req.create_entity('Card', title=u"Venus",
                                      content=u"a planet")
            breadcrumbs = venus.cw_adapt_to('IBreadCrumbs').breadcrumbs()
            self.assertListEqual([venus], breadcrumbs)

    def test_onecard(self):
        with self.admin_access.web_request() as req:
            venus = req.create_entity('Card', title=u"Venus",
                                      content=u"a planet", wikiid=u"Venus")
            breadcrumbs = venus.cw_adapt_to('IBreadCrumbs').breadcrumbs()
            self.assertListEqual([venus], breadcrumbs)

    def test_twocards(self):
        """Planet and Planet/Venus"""
        with self.admin_access.web_request() as req:
            planet = req.create_entity('Card', title=u"Planet",
                                       wikiid=u"Planet")
            venus = req.create_entity('Card', title=u"Venus",
                                      wikiid=u"Planet/Venus")
            breadcrumbs = venus.cw_adapt_to('IBreadCrumbs').breadcrumbs()
            self.assertListEqual([planet, venus], breadcrumbs)

    def test_onecardnested(self):
        """Planet/Venus without Planet"""
        with self.admin_access.web_request() as req:
            venus = req.create_entity('Card', title=u"Venus",
                                      wikiid=u"Planet/Venus")
            breadcrumbs = venus.cw_adapt_to('IBreadCrumbs').breadcrumbs()
            self.assertListEqual(['Planet', venus], breadcrumbs[1:])

    def test_treecards_oneinexistent(self):
        """Astronomy and Astronomy/Planet/Venus (no Astronomy/Planet)"""
        with self.admin_access.web_request() as req:
            astro = req.create_entity('Card', title=u"Astronomy",
                                      wikiid=u"Astronomy")
            venus = req.create_entity('Card', title=u"Venus",
                                      wikiid=u"Astronomy/Planet/Venus")
            breadcrumbs = venus.cw_adapt_to('IBreadCrumbs').breadcrumbs()
            self.assertListEqual([astro, 'Planet', venus], breadcrumbs)

    def test_treecards_oneinexistent2(self):
        """Astronomy/Planet and Astronomy/Planet/Venus (no Astronomy)"""
        with self.admin_access.web_request() as req:
            planet = req.create_entity('Card', title=u"Planets",
                                       wikiid=u"Astronomy/Planet")
            venus = req.create_entity('Card', title=u"Venus",
                                      wikiid=u"Astronomy/Planet/Venus")
            breadcrumbs = venus.cw_adapt_to('IBreadCrumbs').breadcrumbs()
            self.assertListEqual(['Astronomy', planet, venus], breadcrumbs[1:])


if __name__ == '__main__':
    unittest_main()




from docutils import nodes, utils
from docutils.parsers.rst.roles import register_canonical_role, set_classes

def card_reference_role(role, rawtext, text, lineno, inliner,
                       options={}, content=[]):
    text = text.strip()
    try:
        wikiid, rest = text.split(u':', 1)
    except:
        wikiid, rest = text, text
    context = inliner.document.settings.context
    ref = context._cw.build_url('card/' + wikiid)
    rset = context._cw.execute('Card C WHERE C wikiid %(w)s', {'w': wikiid})
    set_classes(options)
    if not rset:
        options['classes'] = ['doesnotexist']
    else:
        options.pop('classes', None)
    return [nodes.reference(rawtext, utils.unescape(rest), refuri=ref,
                            **options)], []

register_canonical_role('card', card_reference_role)




from cubicweb.predicates import match_form_params
from cubicweb.web.views import basecontrollers


class CardViewController(basecontrollers.ViewController):
    __regid__ = 'card'
    __select__ = (basecontrollers.ViewController.__select__ &
                  match_form_params('wikiid'))

    def publish(self, rset=None):
        rset = self._cw.execute('Card C WHERE C wikiid %(wikiid)s',
                                self._cw.form)
        if rset:
            # Drop wikiid from form to get the standard behaviour.
            # Otherwise, the presence of wikiid in form ensures that
            # CardDoesNotExistView is selected.
            self._cw.form.pop('wikiid')
        return super(CardViewController, self).publish(rset)



from cubicweb.devtools import buildconfig, loadconfig
from cubicweb.devtools.testlib import RealDBTest

def setup_module(options):
    if options.source:
        configcls = loadconfig(options.source)
    elif options.dbname is None:
        raise Exception('either <sourcefile> or <dbname> options are required')
    else:
        configcls = buildconfig(options.dbuser, options.dbpassword,
                                               options.dbname, options.euser,
                                               options.epassword)
    RealDatabaseTC.configcls = configcls

class RealDatabaseTC(RealDBTest):
    configcls = None # set by setup_module()

    def test_all_primaries(self):
        for rset in self.iter_individual_rsets(limit=50):
            yield self.view, 'primary', rset, rset.req.reset_headers()
    
    ## startup views
    def test_startup_views(self):
        for vid in self.list_startup_views():
            req = self.request()
            yield self.view, vid, None, req


if __name__ == '__main__':
    from logilab.common.testlib import unittest_main
    unittest_main()



"""this contains the cube-specific entities' classes"""

from cubicweb.entities import AnyEntity, fetch_config


class Card(AnyEntity):
    """customized class for Card entities"""
    __regid__ = 'Card'
    rest_attr = 'wikiid'

    fetch_attrs, cw_fetch_order = fetch_config(['title'])

    def dc_title(self):
        if self.wikiid:
            return self.wikiid.split('/')[-1]
        else:
            return self.title

    def dc_description(self, format='text/plain'):
        return self.printable_value('synopsis', format=format)

    def rest_path(self):
        if self.wikiid:
            return '%s/%s' % (str(self.e_schema).lower(),
                              self._cw.url_quote(self.wikiid, safe='/'))
        else:
            return super(Card, self).rest_path()
